# config.py
#
# (c) Copyright 2010 Michael Towers (larch42 at googlemail dot com)
#
# This file is part of the larch project.
#
#    larch is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    larch is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with larch; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#----------------------------------------------------------------------------
# 2010.07.13

# Basic (default) configuration information for larch
# Some of these can be changed ...????

# Default volume label
LABEL = 'LARCH-7.2'
# Default iso file
ISOFILE = 'larch7.iso'
# Default volume label for boot iso
ISOLABEL = 'LARCH-7-BOOT'
# Default boot iso file
BOOTISO = 'larch7boot.iso'
# Used as a mount point
MP = '/tmp/larch_mnt'
# Mount point (within chroot) for bind-mounting iso build directory
ISOBUILDMNT = '/tmp/isodir_mnt'
# Mount point for medium sources
SOURCEMOUNT = '/tmp/larch_mntsrc'
# Temporary directory for building medium boot directory
BUILD0 = '/tmp/larch_build'

SYSLINUXDIR = '/usr/lib/syslinux'
GRUBDIR = '/usr/lib/grub/i386-pc'

# A customized pacman.conf file is used for the installation, generated
# dynamically according to the options and the profile.
PACMAN_CONF = '/tmp/larch_pacman.conf'

# Medium detection alternatives
detection_methods = 'label|uuid|device|search'

# Some basic paths
import os, sys
module_dir = os.path.dirname(os.path.realpath(__file__))
base_dir = os.path.dirname(module_dir)
script_dir = base_dir + '/buildscripts'

# File to prevent two instances of larch from running
LOCKFILE = '/tmp/larch_lock'
# File (stem) for log
LOGFILE = '/tmp/larch_log_'
# The path to the Arch installation which is to be larchified
INSTALLATION = '/home/larchbuild'

# These paths are intended for use in 'chroot installation_dir', etc.
# The base directory of the larchified stuff
CHROOT_DIR_BUILD = '/.larch'
# This is the base of all stuff to be cleared on a rerun of larchify
CHROOT_DIR_LARCHIFY = CHROOT_DIR_BUILD + '/larchify'
# The base directory of the medium building area
CHROOT_DIR_MEDIUM = CHROOT_DIR_LARCHIFY + '/medium'
# Area for building the (mods.sqf) overlay
CHROOT_DIR_OVERLAY = CHROOT_DIR_LARCHIFY + '/overlay'
# Location for saving the system.sqf (outside of the larchify area)
CHROOT_SYSTEMSQF = CHROOT_DIR_BUILD + '/system.sqf'

def debug(text):
    sys.stderr.write("DEBUG: " + text.strip() + "\n")
    sys.stderr.flush()

def start_translator(switchC=True):
    import gettext, __builtin__
    gettext.install('larch', base_dir+'/i18n', unicode=1)
    __builtin__.lang = (os.environ.get("LANGUAGE") or os.environ.get("LC_ALL")
                or os.environ.get("LC_MESSAGES") or os.environ.get("LANG"))
    if switchC:
        # If subprocesses must be run without i18n because the text
        # output is parsed.
        os.environ["LANGUAGE"] = "C"

